import 'package:hub/backend/api_requests/api_calls.dart';
import 'package:hub/backend/api_requests/api_manager.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:hub/flutter_flow/random_data_util.dart';
import 'package:hub/pages/home_page/home_page_model.dart';
import 'package:hub/shared/helpers/db_helper.dart';
import 'package:hub/shared/utils/device_util.dart';
import 'package:hub/shared/utils/dialog_util.dart';
import 'package:hub/shared/utils/log_util.dart';

import '/flutter_flow/flutter_flow_util.dart';
import 'sign_in_template_component_widget.dart'
    show SignInTemplateComponentWidget;
import 'package:flutter/material.dart';

class SignInTemplateComponentModel
    extends FlutterFlowModel<SignInTemplateComponentWidget> {
  ///  State fields for stateful widgets in this component.

  final formKey = GlobalKey<FormState>();
  final unfocusNode = FocusNode();
  // State field(s) for emailAddress widget.
  FocusNode? emailAddressFocusNode;
  TextEditingController? emailAddressTextController;
  String? Function(BuildContext, String?)? emailAddressTextControllerValidator;

  SignInTemplateComponentModel();
  String? _emailAddressTextControllerValidator(
      BuildContext context, String? val) {
    if (val == null || val.isEmpty) {
      return FFLocalizations.of(context).getText(
        'xhnawzcb' /* Campo é necessário */,
      );
    }

    if (!RegExp(kTextValidatorEmailRegex).hasMatch(val)) {
      return FFLocalizations.of(context).getText(
        's3j1hjqx' /* E-mail Inv */,
      );
    }
    return null;
  }

  // State field(s) for password widget.
  FocusNode? passwordFocusNode;
  TextEditingController? passwordTextController;
  late bool passwordVisibility;
  String? Function(BuildContext, String?)? passwordTextControllerValidator;
  String? _passwordTextControllerValidator(BuildContext context, String? val) {
    if (val == null || val.isEmpty) {
      return FFLocalizations.of(context).getText(
        '9cs5wlmc' /* Campo é necessário */,
      );
    }

    return null;
  }

  Future<bool> checkLocals({
    String? cliUUID,
    required BuildContext context,
    required FlutterFlowModel model,
  }) async {
    final GetLocalsCall callback = PhpGroup.getLocalsCall;

    final response = await callback.call();

    if (response.jsonBody['error']) {
      DialogUtil.errorDefault(context);
      return false;
    }
    List<dynamic> locals = response.jsonBody['locais'] ?? [];

    if (locals.isEmpty) {
      return false;
    } else {
      return true;
    }
  }

  Future signInLoginAction(
    BuildContext context,
    FlutterFlowModel model, {
    String? emailAdress,
    String? password,
  }) async {
    try {
      final ApiCallResponse? response;
      final DatabaseHelper db = DatabaseHelper();
      final LoginCall callback = PhpGroup.loginCall;
      AppState().deviceDescription = randomString(
        10,
        10,
        true,
        false,
        false,
      );

      final String? devUUID;
      final String userUUID;
      final String status;
      final String userDevUUID;
      final String userName;
      final String email;
      final String passwd;
      final bool isLogged;

      email = emailAdress!;
      passwd = password!;

      devUUID = await DeviceUtil.getDevUUID();

      if ((email != '') && (passwd != '')) {
        AppState().email = email;
        AppState().passwd = passwd;
        response = await callback.call();

        if (response.jsonBody['error'] == false) {
          userUUID = response.jsonBody['uid'];

          status = response.jsonBody['user']['status'];
          userDevUUID = response.jsonBody['user']['dev_id'];
          userName = response.jsonBody['user']['name'];

          db.update('devUUID', devUUID, 'user');
          db.update('userUUID', userUUID, 'user');
          db.update('userDevUUID', userDevUUID, 'user');
          db.update('status', status, 'user');
          db.update('userName', userName, 'user');

          isLogged = true;
          await checkLocals(context: context, model: model).then((value) {
            AppState().haveLocal = value;
            AppState().isLogged = isLogged;
            AppState().update(() {});
            toggleApp(context);
          });
        } else {
          if (response.jsonBody['error'] == null) {
            DialogUtil.errorDefault(context);
          } else {
            DialogUtil.error(
                context, response.jsonBody['error_msg'].toString());
          }
        }
      }

      return;
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed(
          'login.php', emailAdress.toString(), "Login", e, s);
    }
  }

  Future toggleApp(BuildContext context) async {
    final haveLocal = AppState().haveLocal;
    if (haveLocal == true) {
      context.go('/homePage');
    } else if (haveLocal == false) {
      context.go(
        '/receptionPage',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: true,
            transitionType: PageTransitionType.fade,
          )
        },
      );
    }
  }

  @override
  void initState(BuildContext context) {
    emailAddressTextControllerValidator = _emailAddressTextControllerValidator;
    passwordVisibility = false;
    passwordTextControllerValidator = _passwordTextControllerValidator;
  }

  @override
  void dispose() {
    emailAddressFocusNode?.dispose();
    emailAddressTextController?.dispose();

    passwordFocusNode?.dispose();
    passwordTextController?.dispose();
  }
}
